; Z88 CamelForth
; camelini.s (Z88 Initialisation & Reset routines)

	module camel

	include	"#packages.def"

        ORG     $C000     ; ROM start in segment 3

; We store some information here, so the application creation utilities
; can access it when making Forth package clients.

	defw	rejoincfentry	; where to enter the code
	defw	cfver		; our version
	defw	cam_dor		; where our DOR is

; The reset routines for Z88
; These are entered from jumps in the card header - any application which
; is a client of CamelForth must have their entry point in the card header
; as well.

; CamelForth only enters here

.mainentry
IF CF_DEBUG
	call	$8000		; invoke Intuition from segment 2
ENDIF
        ld      hl,clients
        inc     (hl)            ; even standard CF is a client
        ld      hl,stduinit

; Applications join the code here

.rejoincfentry
        ld      (uinitadd),hl
	push	bc		; ensure extra dummy cell on stack
	ld	(initsp),sp	; save initial parameter SP
                                ; (left in segment 0 as initialised by OZ)
	jp	COLD		; enter top-level Forth word


; Application DOR

defc    in_bank =     $3F

.cam_dor        defb      0,0,0   ; links to parent, brother, son
	defb	0,0,0
	defw	pkg_block	; we've got a package
	defb	in_bank
	defb	$83	; DOR type - application
	defb	indorend-indorstart
.indorstart     defb      '@'     ; key to info section
	defb	ininfend-ininfstart
.ininfstart     defw      0
	defb	'C'	; application key
IF CF_DEBUG
	defb	95	; 23.75K contigious RAM for debug builds
ELSE
	defb	0	; default contigious RAM (8K or 40K)
ENDIF
	defw	0	; overhead
        defw    0       ; unsafe workspace
     	defw   	0	; safe workspace
	defw	fake_entry
IF CF_DEBUG
	defb	0	; bank bindings
	defb	in_bank	; if only 8K RAM, segment 2 is ROM
	defb	in_bank-1	; Intuition debugger
	defb	in_bank	; CamelForth
ELSE
	defb	0	; bank bindings
	defb	in_bank	; if only 8K RAM, segments 2 & 3 are ROM
	defb	in_bank	; to help prevent memory corruption
	defb	in_bank
ENDIF
	defb	$22	; BAD application, preserve screen on preemption
	defb	1	; caps lock ON
.ininfend       defb      'H'     ; key to help section
	defb	inhlpend-inhlpstart
.inhlpstart     defw      cam_dor ; no topics
	defb	in_bank
	defw	cam_dor		; no commands
	defb	in_bank
	defw	in_help		; help screen
	defb	in_bank
	defw	cam_dor		; no tokens
	defb	in_bank
.inhlpend       defb      'N'     ; key to name section
	defb	innamend-innamstart
IF CF_DEBUG
.innamstart     defm      "CamelDebug"&0
ELSE
.innamstart	defm	  "CamelForth"&0
ENDIF
.innamend       defb      $FF
.indorend


; Help entries

.in_help        defm      $7F&$7F
.versionmsg     defm	"Z88 CamelForth v"
	defm	((cfver/$100)+'0')&"."&(((cfver/$10)~$f)+'0')&((cfver~$f)+'0')&" "
.endversionmsg
	include	"camelver.def"
	defb	0


; Package Information Block

.pkg_block
	defb	$1E		; package ID
	defm	"P"
        defb      $0E             ; highest routine number

	defw	call_inf	; fth_inf (00)
	defw	call_ayt	; fth_ayt (02)
	defw	call_bye	; fth_bye (04)
	defw	call_dat	; fth_dat (06)
	defw	call_exp	; fth_exp (08)

	defw	call_in		; fth_in (0a)
	defw	call_reg	; fth_reg (0c)
	defw	call_drg	; fth_drg (0e)

; The standard package calls

defc    seg3    =     $04D3
defc    seg3p   =     $0D3
defc	seg2    =     $04D2
defc	segp2   =     $0D2
defc    clients =     $04EA           ; our byte in segment 0
defc	datbank =     $04E9           ; our other byte

.call_inf
	ld	a,(seg3)
	ld	b,a	
	ld	hl,inf_mess	; BHL=package name
	ld	c,$01		; need version 1.0
        ld      de,cfver	; our version number
.call_ayt
	and	a		; plug in call_ayt here, as always okay
	ret

.inf_mess
	defm	"Forth"&0

.call_bye
	ld	a,(clients)
	and	a
	jr	z,byeok		; continue if no current clients
	ld	a,rc_use
	scf			; else "in use" error
	ret
.byeok	ld	a,(datbank)
	and	a
	ret	z		; success if no bank allocated
	call_pkg(pkg_bfr)	; else deallocate (Packages must be available
				; when _bye is called, so this always succeeds)
	xor	a		; "success"
	ld	(datbank),a	; and clear away bank
	ret

.call_dat
	ld	bc,0
	ld	d,b
	ld	e,b		; no resources used
	ld	a,(datbank)
	and	a
	jr	z,datok		; valid if no bank allocated
	ex	af,af'
	ld	a,(seg2)
	ex	af,af'
	ld	(seg2),a
	out	(segp2),a	; bind it in
	push	hl
	ld	de,($8002)	; DE=bytes "let out"
	ld	hl,16384
	and	a
	sbc	hl,de
	ex	de,hl		; DE=bytes in use by Forth itself
	pop	hl
	ex	af,af'
	ld	(seg2),a
	out	(segp2),a	; rebind seg2
.datok	ld	a,(clients)	; number of clients
	and	a
	ret

.call_exp
	ld	a,rc_unk
	scf
	ret


; Registration of client applications (BYE deals with deregistration)
; This is now moved to romheader.asm to save space

; Registration and deregistration of client packages (they do most of the
; work - we do the bare minimum here)

.call_reg
	ld	hl,datbank
	ld	a,(hl)
	and	a
	jr	nz,regok	; okay if bank already available
	ld	a,bnk_even
	call_pkg(pkg_bal)	; get an even bank
	ret	c		; exit if error
	ld	(hl),a		; store it
	ld	a,(seg2)
	ex	af,af'		; save current seg2
	ld	a,(hl)
	ld	(seg2),a
	out	(segp2),a	; bind our new bank in
	push	hl
	ld	hl,$8000	; start of seg2
	xor	a
	ld	b,a
	ld	(hl),3		; data layout version "3"
.bnkclr	inc	hl
	ld	(hl),a
	djnz	bnkclr		; clear the next 256 bytes
	pop	hl
	ex	af,af'
	ld	(seg2),a
	out	(segp2),a	; restore seg2 binding
.regok	inc	hl
	inc	(hl)		; increment number of clients
	ret			; carry will be clear at this point

.call_drg
	ld	hl,clients
	dec	(hl)		; decrement number of clients
	and	a
	ret
